@@ -182,7 +182,7 @@ def print_help():
     print "  -s  --server <srv:port>  Listen on server:port [*]"
     print "  -t  --templates <templ>  Template directory [*]"
     print
-    print "  -l  --logging <0..2>     Set logging level (-1=off, 0= least, 2= most) [*]"
+    print "  -l  --logging <-1..2>    Set logging level (-1=off, 0= least, 2= most) [*]"
     print "  -w  --weblogging         Enable cherrypy access logging"
     print
     print "  -b  --browser <0..1>     Auto browser launch (0= off, 1= on) [*]"
@@ -209,7 +209,7 @@ def print_help():
     print "      --new                Run a new instance of SABnzbd"
     print ""
     print "NZB (or related) file:"
-    print "  NZB or zipped NZB file, with extension .nzb, .zip, .rar, .gz, or .bz2"
+    print "  NZB or compressed NZB file, with extension .nzb, .zip, .rar, .7z, .gz, or .bz2"
     print ""
 
 
@@ -502,7 +502,7 @@ def all_localhosts():
 def check_resolve(host):
     """ Return True if 'host' resolves """
     try:
-        dummy = socket.getaddrinfo(host, None)
+        socket.getaddrinfo(host, None)
     except:
         # Does not resolve
         return False
@@ -598,7 +598,7 @@ def get_webhost(cherryhost, cherryport, https_port):
             cherryhost = cherryhost.strip('[]')
         else:
             try:
-                info = socket.getaddrinfo(cherryhost, None)
+                socket.getaddrinfo(cherryhost, None)
             except:
                 cherryhost = cherryhost.strip('[]')
 
@@ -659,12 +659,12 @@ def attach_server(host, port, cert=None, key=None, chain=None):
 def is_sabnzbd_running(url):
     """ Return True when there's already a SABnzbd instance running. """
     try:
-        url = '%s&mode=version' % (url)
+        url = '%s&mode=version' % url
         # Do this without certificate verification, few installations will have that
         prev = sabnzbd.set_https_verification(False)
         ver = get_from_url(url)
         sabnzbd.set_https_verification(prev)
-        return (ver and (re.search(r'\d+\.\d+\.', ver) or ver.strip() == sabnzbd.__version__))
+        return ver and (re.search(r'\d+\.\d+\.', ver) or ver.strip() == sabnzbd.__version__)
     except:
         return False
 
@@ -728,7 +728,7 @@ def evaluate_inipath(path):
             return path
 
 
-def commandline_handler(frozen=True):
+def commandline_handler():
     """ Split win32-service commands are true parameters
         Returns:
             service, sab_opts, serv_opts, upload_nzbs
@@ -829,7 +829,6 @@ def main():
     vista_plus = False
     win64 = False
     repair = 0
-    api_url = None
     no_login = False
     sabnzbd.RESTART_ARGS = [sys.argv[0]]
     pid_path = None
@@ -865,9 +864,9 @@ def main():
         elif opt in ('-b', '--browser'):
             try:
                 autobrowser = bool(int(arg))
-            except:
+            except ValueError:
                 autobrowser = True
-        elif opt in ('--autorestarted', ):
+        elif opt == '--autorestarted':
             autorestarted = True
         elif opt in ('-c', '--clean'):
             clean_up = True
@@ -886,36 +885,36 @@ def main():
             exit_sab(0)
         elif opt in ('-p', '--pause'):
             pause = True
-        elif opt in ('--https',):
+        elif opt == '--https':
             https_port = int(arg)
             sabnzbd.RESTART_ARGS.append(opt)
             sabnzbd.RESTART_ARGS.append(arg)
-        elif opt in ('--repair',):
+        elif opt == '--repair':
             repair = 1
             pause = True
-        elif opt in ('--repair-all',):
+        elif opt == '--repair-all':
             repair = 2
             pause = True
-        elif opt in ('--log-all',):
+        elif opt == '--log-all':
             sabnzbd.LOG_ALL = True
-        elif opt in ('--disable-file-log'):
+        elif opt == '--disable-file-log':
             no_file_log = True
-        elif opt in ('--no-login',):
+        elif opt == '--no-login':
             no_login = True
-        elif opt in ('--pid',):
+        elif opt == '--pid':
             pid_path = arg
             sabnzbd.RESTART_ARGS.append(opt)
             sabnzbd.RESTART_ARGS.append(arg)
-        elif opt in ('--pidfile',):
+        elif opt == '--pidfile':
             pid_file = arg
             sabnzbd.RESTART_ARGS.append(opt)
             sabnzbd.RESTART_ARGS.append(arg)
-        elif opt in ('--new',):
+        elif opt == '--new':
             new_instance = True
-        elif opt in ('--console',):
+        elif opt == '--console':
             sabnzbd.RESTART_ARGS.append(opt)
             osx_console = True
-        elif opt in ('--ipv6_hosting',):
+        elif opt == '--ipv6_hosting':
             ipv6_hosting = arg
 
     sabnzbd.MY_FULLNAME = os.path.normpath(os.path.abspath(sabnzbd.MY_FULLNAME))
@@ -1006,13 +1005,13 @@ def main():
         if enable_https and https_port:
             try:
                 cherrypy.process.servers.check_port(cherryhost, https_port, timeout=0.05)
-            except IOError, error:
+            except IOError:
                 Bail_Out(browserhost, cherryport)
             except:
                 Bail_Out(browserhost, cherryport, '49')
         try:
             cherrypy.process.servers.check_port(cherryhost, cherryport, timeout=0.05)
-        except IOError, error:
+        except IOError:
             Bail_Out(browserhost, cherryport)
         except:
             Bail_Out(browserhost, cherryport, '49')
@@ -1049,7 +1048,7 @@ def main():
                         else:
                             # In case HTTPS == HTTP port
                             cherryport = newport
-                            sabnzbd.cfg.port.set(newport)
+                            sabnzbd.cfg.cherryport.set(newport)
         except:
             # Something else wrong, probably badly specified host
             Bail_Out(browserhost, cherryport, '49')
@@ -1180,7 +1179,7 @@ def main():
         logging.info('Preferred encoding = ERROR')
         preferredencoding = ''
 
-    # On Linux/FreeBSD/Unix "UTF-8" is strongly, strongly adviced:
+    # On Linux/FreeBSD/Unix "UTF-8" is strongly, strongly advised:
     if not sabnzbd.WIN32 and not sabnzbd.DARWIN and not ('utf' in preferredencoding.lower() and '8' in preferredencoding.lower()):
         logging.warning(T("SABnzbd was started with encoding %s, this should be UTF-8. Expect problems with Unicoded file and directory names in downloads.") % preferredencoding)
 
@@ -1237,8 +1236,6 @@ def main():
 
     if autobrowser is not None:
         sabnzbd.cfg.autobrowser.set(autobrowser)
-    else:
-        autobrowser = sabnzbd.cfg.autobrowser()
 
     if not sabnzbd.WIN_SERVICE and not getattr(sys, 'frozen', None) == 'macosx_app':
         signal.signal(signal.SIGINT, sabnzbd.sig_handler)
@@ -1601,7 +1598,7 @@ if sabnzbd.WIN32:
             win32serviceutil.ServiceFramework.__init__(self, args)
 
             self.hWaitStop = win32event.CreateEvent(None, 0, 0, None)
-            self.overlapped = pywintypes.OVERLAPPED()  # @UndefinedVariable
+            self.overlapped = pywintypes.OVERLAPPED()
             self.overlapped.hEvent = win32event.CreateEvent(None, 0, 0, None)
             sabnzbd.WIN_SERVICE = self
 
