@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2018 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,7 +22,6 @@ sabnzbd.bpsmeter - bpsmeter
 import time
 import logging
 import re
-from math import floor
 
 import sabnzbd
 from sabnzbd.constants import BYTES_FILE_NAME, BYTES_FILE_NAME_OLD, KIBI
@@ -74,18 +73,6 @@ def last_month_day(tm):
     return day
 
 
-def this_month_day(t=None):
-    """ Return current day of the week, month 1..31 """
-    t = t or time.localtime(t)
-    return time.localtime(t).tm_mday
-
-
-def this_week_day(t=None):
-    """ Return current day of the week 1..7 """
-    t = t or time.localtime(t)
-    return time.localtime(t).tm_wday + 1
-
-
 def next_month(t):
     """ Return timestamp for start of next month """
     now = time.localtime(t)
@@ -131,6 +118,9 @@ class BPSMeter(object):
         self.month_total = {}
         self.grand_total = {}
 
+        self.timeline_total = {}
+
+        self.day_label = time.strftime("%Y-%m-%d")
         self.end_of_day = tomorrow(t)      # Time that current day will end
         self.end_of_week = next_week(t)    # Time that current day will end
         self.end_of_month = next_month(t)  # Time that current month will end
@@ -149,14 +139,13 @@ class BPSMeter(object):
         data = (self.last_update, self.grand_total,
                 self.day_total, self.week_total, self.month_total,
                 self.end_of_day, self.end_of_week, self.end_of_month,
-                self.quota, self.left, self.q_time
+                self.quota, self.left, self.q_time, self.timeline_total
                )
         sabnzbd.save_admin(data, BYTES_FILE_NAME)
 
     def defaults(self):
         """ Get the latest data from the database and assign to a fake server """
         logging.debug('Setting default BPS meter values')
-        import sabnzbd.database
         history_db = sabnzbd.database.HistoryDB()
         grand, month, week = history_db.get_history_size()
         history_db.close()
@@ -185,12 +174,15 @@ class BPSMeter(object):
             self.last_update, self.grand_total, \
                 self.day_total, self.week_total, self.month_total, \
                 self.end_of_day, self.end_of_week, self.end_of_month = data[:8]
-            if len(data) == 11:
-                self.quota, self.left, self.q_time = data[8:]
+            if len(data) >= 11:
+                self.quota, self.left, self.q_time = data[8:11]
                 logging.debug('Read quota q=%s l=%s reset=%s',
                               self.quota, self.left, self.q_time)
                 if abs(quota - self.quota) > 0.5:
                     self.change_quota()
+                # Get timeline stats
+                if len(data) == 12:
+                    self.timeline_total = data[11]
             else:
                 self.quota = self.left = cfg.quota_size.get_float()
             res = self.reset_quota()
@@ -213,6 +205,7 @@ class BPSMeter(object):
             t = time.time()
         if t > self.end_of_day:
             # current day passed. get new end of day
+            self.day_label = time.strftime("%Y-%m-%d")
             self.day_total = {}
             self.end_of_day = tomorrow(t) - 1.0
 
@@ -241,13 +234,18 @@ class BPSMeter(object):
                 self.grand_total[server] = 0L
             self.grand_total[server] += amount
 
+            if server not in self.timeline_total:
+                self.timeline_total[server] = {}
+            if self.day_label not in self.timeline_total[server]:
+                self.timeline_total[server][self.day_label]= 0L
+            self.timeline_total[server][self.day_label] += amount
+
             # Quota check
             if self.have_quota and self.quota_enabled:
                 self.left -= amount
                 if self.left <= 0.0:
-                    from sabnzbd.downloader import Downloader
-                    if Downloader.do and not Downloader.do.paused:
-                        Downloader.do.pause()
+                    if sabnzbd.downloader.Downloader.do and not sabnzbd.downloader.Downloader.do.paused:
+                        sabnzbd.downloader.Downloader.do.pause()
                         logging.warning(T('Quota spent, pausing downloading'))
 
         # Speedometer
@@ -270,10 +268,9 @@ class BPSMeter(object):
             logging.debug("bps: %s", self.bps)
             self.log_time = t
 
-        refresh_rate = cfg.refresh_rate() if cfg.refresh_rate() else 1.0
-        if self.speed_log_time < (t - float(refresh_rate)):
+        if self.speed_log_time < (t - 1.0):
             self.add_empty_time()
-            self.bps_list.append("%i" % (self.bps / KIBI))
+            self.bps_list.append(int(self.bps / KIBI))
             self.speed_log_time = t
 
     def reset(self):
@@ -284,13 +281,12 @@ class BPSMeter(object):
         self.bps = 0.0
 
     def add_empty_time(self):
-        refresh_rate = cfg.refresh_rate() if cfg.refresh_rate() else 1.0
-        time_diff = time.time() - self.speed_log_time
-        nr_diffs = int(floor(time_diff / refresh_rate))
-
+        # Extra zeros, but never more than the maxium!
+        nr_diffs = min(int(time.time() - self.speed_log_time), self.bps_list_max)
         if nr_diffs > 1:
-            self.bps_list.extend(['0.0'] * nr_diffs)
+            self.bps_list.extend([0] * nr_diffs)
 
+        # Always trim the list to the max-length
         if len(self.bps_list) > self.bps_list_max:
             self.bps_list = self.bps_list[len(self.bps_list) - self.bps_list_max:]
 
@@ -307,7 +303,8 @@ class BPSMeter(object):
         return self.grand_total.get(server, 0L), \
                self.month_total.get(server, 0L), \
                self.week_total.get(server, 0L),  \
-               self.day_total.get(server, 0L)
+               self.day_total.get(server, 0L), \
+               self.timeline_total.get(server, {})
 
     def clear_server(self, server):
         """ Clean counters for specified server """
@@ -319,14 +316,44 @@ class BPSMeter(object):
             del self.month_total[server]
         if server in self.grand_total:
             del self.grand_total[server]
+        if server in self.timeline_total:
+            del self.timeline_total[server]
         self.save()
 
     def get_bps(self):
         return self.bps
 
     def get_bps_list(self):
+        refresh_rate = int(cfg.refresh_rate()) if cfg.refresh_rate() else 1
         self.add_empty_time()
-        return self.bps_list
+        # We record every second, but display at the user's refresh-rate
+        return self.bps_list[::refresh_rate]
+
+    def get_stable_speed(self, timespan=10):
+        """ See if there is a stable speed the last <timespan> seconds
+            None: indicates it can't determine yet
+            False: the speed was not stable during <timespan>
+        """
+        if len(self.bps_list) < timespan:
+            return None
+
+        # Calculate the variance in the speed
+        avg = sum(self.bps_list[-timespan:]) / timespan
+        vari = 0
+        for bps in self.bps_list[-timespan:]:
+            vari += abs(bps - avg)
+        vari = vari / timespan
+
+        try:
+            # See if the variance is less than 5%
+            if (vari / (self.bps / KIBI)) < 0.05:
+                return avg
+            else:
+                return False
+        except:
+            # Probably one of the values was 0
+            pass
+        return None
 
     def reset_quota(self, force=False):
         """ Check if it's time to reset the quota, optionally resuming
@@ -440,9 +467,8 @@ class BPSMeter(object):
 
     def resume(self):
         """ Resume downloading """
-        from sabnzbd.downloader import Downloader
-        if cfg.quota_resume() and Downloader.do and Downloader.do.paused:
-            Downloader.do.resume()
+        if cfg.quota_resume() and sabnzbd.downloader.Downloader.do and sabnzbd.downloader.Downloader.do.paused:
+            sabnzbd.downloader.Downloader.do.resume()
 
     def midnight(self):
         """ Midnight action: dummy update for all servers """
