@@ -28,9 +28,10 @@ import logging
 
 import sabnzbd
 import sabnzbd.cfg as cfg
-from sabnzbd.misc import int_conv, clip_path, long_path, remove_all, globber, \
-    format_time_string, has_win_device, real_path, remove_file
+from sabnzbd.misc import int_conv, clip_path, long_path, remove_all, \
+    format_time_string, real_path, remove_file
 from sabnzbd.encoding import TRANS, unicoder
+from sabnzbd.decorators import synchronized
 from sabnzbd.newsunpack import build_command, EXTRACTFROM_RE, EXTRACTED_RE, rar_volumelist
 from sabnzbd.postproc import prepare_extraction_path
 from sabnzbd.utils.rarfile import RarFile
@@ -46,6 +47,10 @@ if sabnzbd.WIN32:
 # Load the regular POpen (which is now patched on Windows)
 from subprocess import Popen
 
+# Need a lock to make sure start and stop is handled correctlty
+# Otherwise we could stop while the thread was still starting
+START_STOP_LOCK = threading.RLock()
+
 MAX_ACTIVE_UNPACKERS = 10
 ACTIVE_UNPACKERS = []
 
@@ -110,6 +115,7 @@ class DirectUnpacker(threading.Thread):
         if none_counter > found_counter:
             self.total_volumes = {}
 
+    @synchronized(START_STOP_LOCK)
     def add(self, nzf):
         """ Add jobs and start instance of DirectUnpack """
         if not cfg.direct_unpack_tested():
@@ -170,10 +176,10 @@ class DirectUnpacker(threading.Thread):
                 break
 
             # Error? Let PP-handle it
-            if linebuf.endswith(('ERROR: ', 'Cannot create', 'in the encrypted file', 'CRC failed', \
-                    'checksum failed', 'You need to start extraction from a previous volume',  \
-                    'password is incorrect', 'Write error', 'checksum error', \
-                    'start extraction from a previous volume')):
+            if linebuf.endswith(('ERROR: ', 'Cannot create', 'in the encrypted file', 'CRC failed',
+                                 'checksum failed', 'You need to start extraction from a previous volume',
+                                 'password is incorrect', 'Write error', 'checksum error',
+                                 'start extraction from a previous volume')):
                 logging.info('Error in DirectUnpack of %s', self.cur_setname)
                 self.abort()
 
@@ -309,6 +315,7 @@ class DirectUnpacker(threading.Thread):
             with self.next_file_lock:
                 self.next_file_lock.wait()
 
+    @synchronized(START_STOP_LOCK)
     def create_unrar_instance(self):
         """ Start the unrar instance using the user's options """
         # Generate extraction path and save for post-proc
@@ -366,9 +373,10 @@ class DirectUnpacker(threading.Thread):
         # Doing the first
         logging.info('DirectUnpacked volume %s for %s', self.cur_volume, self.cur_setname)
 
+    @synchronized(START_STOP_LOCK)
     def abort(self):
         """ Abort running instance and delete generated files """
-        if not self.killed:
+        if not self.killed and self.cur_setname:
             logging.info('Aborting DirectUnpack for %s', self.cur_setname)
             self.killed = True
 
@@ -404,7 +412,6 @@ class DirectUnpacker(threading.Thread):
                     except:
                         # The user will have to remove it themselves
                         logging.info('Failed to clean Direct Unpack after aborting %s', rarfile_nzf.filename, exc_info=True)
-                        pass
                 else:
                     # We can just remove the whole path
                     remove_all(extraction_path, recursive=True)
